/**
 * Approved for Public Release: 10-4800. Distribution Unlimited.
 * Copyright 2011 The MITRE Corporation,
 * Licensed under the Apache License,
 * Version 2.0 (the "License");
 *
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 */

package org.wiredwidgets.cow.server.service;

import java.io.InputStream;
import java.util.List;
import javax.xml.transform.stream.StreamSource;
import org.wiredwidgets.cow.server.api.service.Deployment;
import org.wiredwidgets.cow.server.api.service.ResourceNames;

/**
 *
 * @author JKRANES
 */
public interface ProcessService {

    public static final String BPMN2_EXTENSION = ".bpmn2";
    public static final String JPDL_EXTENSION = ".jpdl.xml";
    public static final String V2_EXTENSION = ".v2.xml";

    /**
     * Creates a new deployment from a StreamSource.  The StreamSource is presumed
     * to contain a JPDL format process definition xml file.
     * @param source
     * @param name the name assigned to the deployment.  The same name, with .jpdl.xml
     * appended, will be used for the resource stored with the deployment.
     * @return
     */
    public Deployment createDeployment(StreamSource source, String name, boolean bpmn2);

    StreamSource getResource(String id, String name);

    void deleteDeployment(String id);

    List<Deployment> findAllDeployments() throws Exception;

    ResourceNames getResourceNames(String deploymentId);

    InputStream getNativeProcessAsStream(String key);

    Deployment getDeployment(String id);

    Deployment saveV2Process(org.wiredwidgets.cow.server.api.model.v2.Process v2Process, String deploymentName);

    InputStream getResourceAsStream(String key, String extension);

    InputStream getResourceAsStreamByDeploymentId(String id, String extension);

    org.wiredwidgets.cow.server.api.model.v2.Process getV2Process(String key);
    
}
